from caboodle.agents.rucaptcha import RucaptchaImageGridAgent
from unittest import mock
import os
import unittest

TRUCKS = None

# Locate data file and read it
for root, dirs, files in os.walk(os.getcwd()):
	if 'commercial_trucks' in files:
		TRUCKS = os.path.join(root, 'commercial_trucks')
		TRUCKS = open(TRUCKS, 'rb').read()

class RucaptchaImageGridAgentTest(unittest.TestCase):
	'''
	Tests the RucaptchaImageGridAgent to verify that it works correctly
	'''

	@classmethod
	def setUpClass(self):
		'''
		Creates a RucaptchaImageGridAgent to test
		'''

		self.agent = RucaptchaImageGridAgent('API KEY', 'USD')

	def test_init(self):
		'''
		Tests that the RucaptchaImageGridAgent initializes correctly
		'''

		self.assertNotEqual(self.agent._cost, 0.0)
		self.assertIsInstance(self.agent._cost, float)

	@mock.patch(
		'caboodle.agents.rucaptcha.lib.get',
		side_effect = lambda *args, **kwags: 'click:1/4/5'
	)
	@mock.patch(
		'caboodle.agents.rucaptcha.lib.post_image_grid',
		side_effect = lambda *args, **kwags: 42
	)
	def test_solve(self, mock_post, mock_get):
		'''
		Tests that the RucaptchaImageGridAgent solves a Challenge correctly
		'''

		if TRUCKS:
			data = {
				'image': TRUCKS,
				'text': 'Select all images with commercial trucks',
				'rows': 3,
				'columns': 3
			}

			self.agent.solve(data)
			self.assertEqual(data['id'], 42)
			self.assertEqual(data['result'], (0, 3, 4))

			with self.assertRaises(TypeError):
				self.agent.solve(None)
		else:
			self.fail('Could not locate data file')

	def test_cost(self):
		'''
		Tests that the RucaptchaImageGridAgent returns the correct cost
		'''

		self.assertNotEqual(self.agent.get_cost(), 0.0)
		self.assertIsInstance(self.agent.get_cost(), float)

	def test_comparable(self):
		'''
		Tests that a RucaptchaImageGridAgent can be compared to another Agent
		'''

		self.assertFalse(self.agent < self.agent)

		with self.assertRaises(TypeError):
			self.agent < None

	def test_success(self):
		'''
		Tests that a RucaptchaImageGridAgent can handle a successful Challenge
		correctly
		'''

		if TRUCKS:
			data = {
				'image': TRUCKS,
				'text': 'Select all images with commercial trucks',
				'rows': 3,
				'columns': 3,
				'id': 42
			}

			self.agent.success(data)
		else:
			self.fail('Could not locate data file')

	def test_fail(self):
		'''
		Tests that a RucaptchaImageGridAgent can handle a failed Challenge
		correctly
		'''

		if TRUCKS:
			data = {
				'image': TRUCKS,
				'text': 'Select all images with commercial trucks',
				'rows': 3,
				'columns': 3,
				'id': 42
			}

			self.agent.fail(data)
		else:
			self.fail('Could not locate data file')

if __name__ == '__main__':
	unittest.main()



'''
Specifications for Challenge solving agents

This module defines abstract classes with compounding levels of functionality.
Every Agent should have a `solve()` and `get_cost()` function and, by default,
will initialize the cost of the Agent to zero. These specifications can not be
instantiated and can only be used by a subclass that implements all of the
required functions. Be sure, however, to explicitly call the `super()` function
from each subclass and from each defined function in that subclass to gain the
functionality of it's parent. See the unit tests for this module for more
information.
'''

from abc import ABCMeta, abstractmethod

class Agent(object, metaclass = ABCMeta):
	'''
	An agent to solve Challenges

	An Agent processes data from a Challenge in the form of a dictionary to
	produce a result. This result is added to the dictionary if and when it
	succeeds. As a general guideline, the result should be appended to the
	dictionary with the key 'result'.
	'''

	def __init__(self):
		self._cost = 0.0

	@abstractmethod
	def solve(self, data):
		'''
		Solves a Challenge and stores the result

		Args:
			data (dict): The Challenge to solve

		Raises:
			TypeError: The Challenge is not a dictionary
			ValueError: The Challenge does not contain the required data
			RuntimeError: The Challenge could not be solved
		'''

		if type(data) is not dict:
			raise TypeError('The data must be a dictionary')

	@abstractmethod
	def get_cost(self):
		'''
		Returns the cost of using this Agent

		Returns:
			The cost as a floating point number
		'''

		return self._cost

	@abstractmethod
	def success(self, data):
		'''
		Performs actions for a successful Challenge

		Args:
			data (dict): The successful Challenge
		'''

		pass

	@abstractmethod
	def fail(self, data):
		'''
		Performs actions for a failed Challenge

		Args:
			data (dict): The failed Challenge
		'''

		pass

	def __lt__(self, other):
		'''
		Compares the cost of this agent to another

		Args:
			other (solver): The other agent to compare

		Returns:
			True if the cost of this agent is less than another

		Raises:
			TypeError: The other agent is not of type Agent
		'''

		if isinstance(other, Agent):
			return self.get_cost() < other.get_cost()
		else:
			raise TypeError('The other agent must be of type Agent')



from caboodle import web
from caboodle.challenges.spec import Challenge
import unittest

class GoodChallenge(Challenge):
	'''
	A complete child of the Challenge specification
	'''

	def __init__(self):
		super().__init__()

	def get_data(self, browser):
		'''
		Collects data needed to solve the Challenge

		Args:
			browser (Browser): The web browser to use

		Returns:
			A dictionary of collected data

		Raises:
			TypeError: The browser is not of type Browser
		'''

		super().get_data(browser)

		return {'text': 'What is the answer to the question?'}

	def submit_data(self, data):
		'''
		Submits the processed data and solves the Challenge

		Args:
			data (dict): The Challenge to submit

		Raises:
			TypeError: The data is not a dictionary
		'''

		super().submit_data(data)

		return data['result']

class BadChallenge(Challenge):
	'''
	An incomplete child of the Challenge specification
	'''

	def __init__(self):
		super().__init__()

class ChallengeTest(unittest.TestCase):
	'''
	Tests the Challenge specification to verify that it works correctly
	'''

	@classmethod
	def setUpClass(self):
		'''
		Creates a Challenge to test
		'''

		self.browser = web.Browser(None)
		self.challenge = GoodChallenge()

	def test_abstraction(self):
		'''
		Tests that an incomplete child of the Challenge specification will
		raise a TypeError
		'''

		with self.assertRaises(TypeError):
			BadChallenge()

	def test_init(self):
		'''
		Tests that the Challenge specification initializes correctly
		'''

		self.assertIsInstance(self.challenge, Challenge)

	def test_get_data(self):
		'''
		Tests that the Challenge specification can collect data
		'''

		data = self.challenge.get_data(self.browser)
		self.assertTrue(data['text'])

		with self.assertRaises(TypeError):
			self.challenge.get_data(None)

	def test_submit_data(self):
		'''
		Tests that the Challenge specification can submit data
		'''

		data = {
			'text': 'What is the answer to the question?',
			'result': 42
		}
		self.assertEqual(self.challenge.submit_data(data), 42)

		with self.assertRaises(TypeError):
			self.challenge.submit_data(None)

	@classmethod
	def tearDownClass(self):
		'''
		Closes the Browser to end the test
		'''

		self.browser.quit()

if __name__ == '__main__':
	unittest.main()



__all__ = [
	'agent', 'browser', 'challenge_utilities', 'challenge', 'local_agent',
	'manual_agent', 'manual_challenge', 'recaptcha_v2_challenge',
	'rucaptcha_image_agent', 'rucaptcha_image_grid_agent', 'rucaptcha_library',
	'solve_media_text_challenge', 'solver'
]



from caboodle.agents.local import LocalAgent
import unittest

class LocalAgentTest(unittest.TestCase):
	'''
	Tests the LocalAgent to verify that it works correctly
	'''

	@classmethod
	def setUpClass(self):
		'''
		Creates a LocalAgent to test
		'''

		self.agent = LocalAgent()

	def test_init(self):
		'''
		Tests that the LocalAgent initializes correctly
		'''

		self.assertEqual(self.agent._cost, 0.0)

	def test_solve(self):
		'''
		Tests that the LocalAgent solves a Challenge correctly
		'''

		data = {'text': 'What is the answer to the question?'}

		self.agent.solve(data)
		self.assertEqual(data['result'], '42')

		with self.assertRaises(TypeError):
			self.agent.solve(None)

	def test_cost(self):
		'''
		Tests that the LocalAgent returns the correct cost
		'''

		self.assertEqual(self.agent.get_cost(), 0.0)

	def test_comparable(self):
		'''
		Tests that a LocalAgent can be compared to another Agent
		'''

		self.assertFalse(self.agent < self.agent)

		with self.assertRaises(TypeError):
			self.agent < None

	def test_success(self):
		'''
		Tests that a LocalAgent can handle a successful Challenge correctly
		'''

		data = {
			'text': 'What is the answer to the question?',
			'result': 42
		}

		self.assertFalse(self.agent.success(data))

	def test_fail(self):
		'''
		Tests that a LocalAgent can handle a failed Challenge correctly
		'''

		data = {
			'text': 'What is the answer to the question?',
			'result': 3.14
		}

		self.assertFalse(self.agent.fail(data))

if __name__ == '__main__':
	unittest.main()



'''
ReCAPTCHA CAPTCHA Challenges

This module is an implementation of the Challenge specification and collects
data to solve ReCAPTCHA CAPTCHAs. To use these Challenges, create a new
instance of them and call the `get_data()` function. Then, process the data
using an Agent and submit it by calling the `submit_data()` function. See the
unit tests for this module for more information.
'''

from caboodle.challenges.spec import Challenge
import caboodle.challenges.util as util
import time

class RecaptchaV2Challenge(Challenge):
	'''
	A Challenge for ReCAPTCHA v2 CAPTCHAs

	Version 2 of ReCAPTCHA is the common "I'm not a robot" CAPTCHA where you are
	prompted to choose a selection of images from a grid. This Challenge will
	locate the grid of images and add the aggregate of all the images to the
	dictionary with the key 'image'. In addition to the CAPTCHA, the elements to
	click, the verify button, the text instructions, the type of image to look
	for, the reload button to get a new CAPTCHA and the dimensions of the image
	grid are added to the dictionary with the keys 'tiles', 'verify', 'text',
	'tag', 'reload', 'columns' and 'rows' respectively.
	'''

	def __init__(self):
		super().__init__()

	def get_data(self, browser):
		'''
		Collects data needed to solve the Challenge

		Args:
			browser (Browser): The web browser to use

		Returns:
			A dictionary of collected data

		Raises:
			TypeError: The browser is not of type Browser
		'''

		super().get_data(browser)

		try:
			data = {}

			# Find widget and switch to it
			browser.switch_to_frame(
				browser.find_element_by_xpath(
					'//iframe[@title="recaptcha widget"]'
				)
			)

			# Start challenge
			browser.find_element_by_id('recaptcha-anchor').click()

			# Return to default frame
			browser.switch_to_default_content()

			# Find challenge and switch to it
			browser.switch_to_frame(
				browser.find_element_by_xpath(
					'//iframe[@title="recaptcha challenge"]'
				)
			)

			# Wait for image to load
			start_time = time.time()
			while len(browser.find_elements_by_tag_name('img')) < 1:
				if time.time() - start_time > 5:
					# Return to default frame
					browser.switch_to_default_content()

					# Find widget and switch to it
					browser.switch_to_frame(
						browser.find_element_by_xpath(
							'//iframe[@title="recaptcha widget"]'
						)
					)

					# Assert challenge is solved
					isChecked = browser.find_element_by_id('recaptcha-anchor')\
						.get_attribute('aria-checked') == 'true'

					if isChecked:
						return 'solved'

					# Return to default frame
					browser.switch_to_default_content()

					# Find challenge and switch to it
					browser.switch_to_frame(
						browser.find_element_by_xpath(
							'//iframe[@title="recaptcha challenge"]'
						)
					)

				time.sleep(1)
			del start_time

			# Get elements
			data['image'] = util.get_image_src(
				browser.find_element_by_tag_name('img')
			)
			data['tiles'] = browser.find_elements_by_class_name(
				'rc-image-tile-target'
			)
			data['verify'] = browser.find_element_by_id(
				'recaptcha-verify-button'
			)
			data['text'] = browser.find_element_by_class_name(
				'rc-imageselect-instructions'
			).text
			data['tag'] = browser.find_element_by_xpath(
				'//div[@class="rc-imageselect-instructions"]//strong'
			).text
			data['reload'] = browser.find_element_by_id(
				'recaptcha-reload-button'
			)
			table = browser.find_element_by_tag_name('table')
			value = table.get_attribute('class')[-2:]
			data['columns'] = int(value[0])
			data['rows'] = int(value[1])

			return data
		except:
			return None

	def submit_data(self, data):
		'''
		Submits the processed data and solves the Challenge

		Args:
			data (dict): The Challenge to submit

		Raises:
			TypeError: The data is not a dictionary
		'''

		super().submit_data(data)

		for tile in data['result']:
			try:
				data['tiles'][tile].click()
			except IndexError:
				pass

		data['verify'].click()



'''
ruCaptcha Challenge solving Agents

This module is an implementation of the Agent specification and solves a
Challenge using the paid online service ruCaptcha. To use these Agents, create a
new instance of one and call the `solve()` function. See the documentation for
each Agent and their respective unit tests for more information.
'''

from caboodle.agents.spec import Agent
import caboodle.libraries.rucaptcha as lib
import time

class RucaptchaImageAgent(Agent):
	'''
	An Agent to solve image based Challenges using ruCaptcha

	Args:
		key (str): The API key to use
		currency (str): The currency to calculate for

	The currency must be a three letter code like USD or EUR.
	'''

	def __init__(self, key, currency = 'USD'):
		super().__init__()

		self._last_update = 0
		self._key = key
		self._currency = currency
		self.get_cost()

	def solve(self, data):
		'''
		Solves a Challenge and stores the result

		Args:
			data (dict): The Challenge to solve

		Raises:
			TypeError: The Challenge is not a dictionary
		'''

		super().solve(data)

		data['id'] = lib.post_image(self._key, data['image'])
		data['result'] = lib.get(self._key, data['id'])

	def get_cost(self):
		'''
		Returns the cost of using this Agent

		Returns:
			The cost as a floating point number
		'''

		super().get_cost()

		if int(time.time()) > self._last_update + 60:
			self._cost = lib.get_rate(self._currency)
			self._last_update = int(time.time())

		return self._cost

	def success(self, data):
		'''
		Performs actions for a successful Challenge

		Args:
			data (dict): The successful Challenge
		'''

		super().success(data)

	def fail(self, data):
		'''
		Performs actions for a failed Challenge

		Args:
			data (dict): The failed Challenge
		'''

		super().fail(data)

		lib.invalid(self._key, data['id'])

class RucaptchaImageGridAgent(Agent):
	'''
	An Agent to solve image grid based Challenges using ruCaptcha

	Args:
		key (str): The API key to use
		currency (str): The currency to calculate for

	The currency must be a three letter code like USD or EUR.
	'''

	def __init__(self, key, currency = 'USD'):
		super().__init__()

		self._last_update = 0
		self._key = key
		self._currency = currency
		self.get_cost()

	def solve(self, data):
		'''
		Solves a Challenge and stores the result

		Args:
			data (dict): The Challenge to solve

		Raises:
			TypeError: The Challenge is not a dictionary
		'''

		super().solve(data)

		data['id'] = lib.post_image_grid(
			self._key,
			data['image'],
			data['text'],
			data['rows'],
			data['columns']
		)
		data['result'] = lib.get(self._key, data['id'])
		data['result'] = data['result'].split(':')[1].split('/')
		data['result'] = tuple([int(i) - 1 for i in data['result']])

	def get_cost(self):
		'''
		Returns the cost of using this Agent

		Returns:
			The cost as a floating point number
		'''

		super().get_cost()

		if int(time.time()) > self._last_update + 60:
			self._cost = lib.get_rate(self._currency, 0.07)
			self._last_update = int(time.time())

		return self._cost

	def success(self, data):
		'''
		Performs actions for a successful Challenge

		Args:
			data (dict): The successful Challenge
		'''

		super().success(data)

	def fail(self, data):
		'''
		Performs actions for a failed Challenge

		Args:
			data (dict): The failed Challenge
		'''

		super().fail(data)

		lib.invalid(self._key, data['id'])



from caboodle.agents.d3cryp7 import d3cryp7TextImageAgent
import os
import unittest

PANGRAM = None
HELLO_WORLD = None
TRUCKS = None

# Locate data file and read it
for root, dirs, files in os.walk(os.getcwd()):
	if 'pangram' in files:
		PANGRAM = os.path.join(root, 'pangram')
		PANGRAM = open(PANGRAM, 'rb').read()

	if 'hello_world' in files:
		HELLO_WORLD = os.path.join(root, 'hello_world')
		HELLO_WORLD = open(HELLO_WORLD, 'rb').read()

	if 'commercial_trucks' in files:
		TRUCKS = os.path.join(root, 'commercial_trucks')
		TRUCKS = open(TRUCKS, 'rb').read()

class d3cryp7TextImageAgentTest(unittest.TestCase):
	'''
	Tests the d3cryp7TextImageAgent to verify that it works correctly
	'''

	@classmethod
	def setUpClass(self):
		'''
		Creates a d3cryp7TextImageAgent to test
		'''

		self.agent = d3cryp7TextImageAgent('http://localhost')

	def test_init(self):
		'''
		Tests that the d3cryp7TextImageAgent initializes correctly
		'''

		self.assertEqual(self.agent._cost, 0.0)
		self.assertIsInstance(self.agent._cost, float)

	def test_solve(self):
		'''
		Tests that the d3cryp7TextImageAgent solves a Challenge correctly
		'''

		if HELLO_WORLD and PANGRAM and TRUCKS:
			data = {
				'image': PANGRAM
			}

			self.agent.solve(data)
			self.assertEqual(
				data['result'],
				'The quick brown fox jumps over the lazy dog.'
			)

			data = {
				'image': HELLO_WORLD
			}

			self.agent.solve(data)
			self.assertEqual(data['result'], 'Hello\nWorld')

			data = {
				'image': TRUCKS
			}

			with self.assertRaises(RuntimeError):
				self.agent.solve(data)

			data = {
				'image': None
			}

			with self.assertRaises(RuntimeError):
				self.agent.solve(data)

			with self.assertRaises(TypeError):
				self.agent.solve(None)
		else:
			self.fail('Could not locate data file')

	def test_cost(self):
		'''
		Tests that the d3cryp7TextImageAgent returns the correct cost
		'''

		self.assertEqual(self.agent.get_cost(), 0.0)
		self.assertIsInstance(self.agent.get_cost(), float)

	def test_comparable(self):
		'''
		Tests that a d3cryp7TextImageAgent can be compared to another Agent
		'''

		self.assertFalse(self.agent < self.agent)

		with self.assertRaises(TypeError):
			self.agent < None

	def test_success(self):
		'''
		Tests that a d3cryp7TextImageAgent can handle a successful Challenge
		correctly
		'''

		if HELLO_WORLD:
			data = {
				'image': HELLO_WORLD
			}

			self.agent.success(data)
		else:
			self.fail('Could not locate data file')

	def test_fail(self):
		'''
		Tests that a d3cryp7TextImageAgent can handle a failed Challenge correctly
		'''

		if PANGRAM:
			data = {
				'image': PANGRAM
			}

			self.agent.fail(data)
		else:
			self.fail('Could not locate data file')

if __name__ == '__main__':
	unittest.main()



'''
d3cryp7 Challenge solving Agents

This module is an implementation of the Agent specification and solves a
Challenge using API calls to a d3cryp7 server. To use this Agent, create a new
instance of it and call the `solve()` function. See the unit tests for this
module for more information.
'''

from caboodle.agents.spec import Agent
from PIL import Image
import caboodle.libraries.d3cryp7 as lib
import time

class d3cryp7TextImageAgent(Agent):
	'''
	An Agent to solve image based Challenges containing text using d3cryp7

	Args:
		url (string): The URL of the d3cryp7 API
		currency (str): The currency to calculate for

	The currency must be a three letter code like USD or EUR.

	You must run a d3cryp7 server in order to use this Agent. You can find
	instructions to do so here:

	https://bitbucket.org/bkvaluemeal/d3cryp7.py
	'''

	def __init__(self, url, currency = 'USD'):
		super().__init__()

		self._last_update = 0
		self._currency = currency
		self._url = url

	def solve(self, data):
		'''
		Solves a Challenge and stores the result

		Args:
			data (dict): The Challenge to solve

		Raises:
			TypeError: The Challenge is not a dictionary
		'''

		super().solve(data)

		data['result'] = lib.recognize_image(self._url, data['image'])

	def get_cost(self):
		'''
		Returns the cost of using this Agent

		Returns:
			The cost as a floating point number
		'''

		super().get_cost()

		if int(time.time()) > self._last_update + 60:
			self._cost = lib.get_rate(self._url, self._currency)['recognize']
			self._last_update = int(time.time())

		return self._cost

	def success(self, data):
		'''
		Performs actions for a successful Challenge

		Args:
			data (dict): The successful Challenge
		'''

		super().success(data)

	def fail(self, data):
		'''
		Performs actions for a failed Challenge

		Args:
			data (dict): The failed Challenge
		'''

		super().fail(data)

class d3cryp7TagImageGridAgent(Agent):
	'''
	An Agent to solve image grid based Challenges using d3cryp7

	Args:
		url (string): The URL of the d3cryp7 API
		currency (str): The currency to calculate for

	The currency must be a three letter code like USD or EUR.

	You must run a d3cryp7 server in order to use this Agent. You can find
	instructions to do so here:

	https://bitbucket.org/bkvaluemeal/d3cryp7.py
	'''

	def __init__(self, url, currency = 'USD'):
		super().__init__()

		self._last_update = 0
		self._currency = currency
		self._url = url

	def solve(self, data):
		'''
		Solves a Challenge and stores the result

		Args:
			data (dict): The Challenge to solve

		Raises:
			TypeError: The Challenge is not a dictionary
		'''

		super().solve(data)

		data['result'] = ()
		results = lib.tag_image_grid(
			self._url,
			data['image'],
			data['columns'],
			data['rows']
		)

		for i in range(len(results)):
			if data['tag'] in results[i]:
				data['result'] += (i,)

	def get_cost(self):
		'''
		Returns the cost of using this Agent

		Returns:
			The cost as a floating point number
		'''

		super().get_cost()

		if int(time.time()) > self._last_update + 60:
			self._cost = lib.get_rate(self._url, self._currency)['tag']
			self._last_update = int(time.time())

		return self._cost

	def success(self, data):
		'''
		Performs actions for a successful Challenge

		Args:
			data (dict): The successful Challenge
		'''

		super().success(data)

	def fail(self, data):
		'''
		Performs actions for a failed Challenge

		Args:
			data (dict): The failed Challenge
		'''

		super().fail(data)



'''
A manually defined Challenge solving Agent

This module is an implementation of the Agent specification and solves a
Challenge using an external function defined by the user. To use this Agent,
create a new instance of it with the desired solving function and call the
`solve()` function. See the unit tests for this module for more information.
'''

from caboodle.agents.spec import Agent

class ManualAgent(Agent):
	'''
	An Agent to solve Challenges with external functions

	Args:
		solve_function (function): The function to solve the Challenge
		success_function (function): The function called on success
		fail_function (function): The function called on fail

	Raises:
		TypeError: An argument is not a function

	This Agent will, by default, have a cost of zero. As a general guideline,
	the solving function should append the result to the dictionary with the
	key 'result'.

	Example:

		lambda data: data.update({'result': 42})
	'''

	def __init__(self, solve_function, success_function, fail_function):
		super().__init__()

		if callable(solve_function):
			self.solve_function = solve_function
		else:
			raise TypeError('The solve function must be a function')

		if callable(success_function):
			self.success_function = success_function
		else:
			raise TypeError('The success function must be a function')

		if callable(fail_function):
			self.fail_function = fail_function
		else:
			raise TypeError('The fail function must be a function')

	def solve(self, data):
		'''
		Solves a Challenge and stores the result

		Args:
			data (dict): The Challenge to solve

		Raises:
			TypeError: The Challenge is not a dictionary
			ValueError: The Challenge does not contain the required data
			RuntimeError: The Challenge could not be solved
		'''

		super().solve(data)

		return self.solve_function(data)

	def get_cost(self):
		'''
		Returns the cost of using this Agent

		Returns:
			The cost as a floating point number
		'''

		super().get_cost()

		return self._cost

	def success(self, data):
		'''
		Performs actions for a successful Challenge

		Args:
			data (dict): The successful Challenge
		'''

		super().success(data)

		return self.success_function(data)

	def fail(self, data):
		'''
		Performs actions for a failed Challenge

		Args:
			data (dict): The failed Challenge
		'''

		super().fail(data)

		return self.fail_function(data)



from caboodle import web
from caboodle.challenges.spec import Challenge
from caboodle.challenges.manual import ManualChallenge
import unittest

class ManualChallengeTest(unittest.TestCase):
	'''
	Tests the ManualChallenge to verify that it works correctly
	'''

	@classmethod
	def setUpClass(self):
		'''
		Creates a Challenge to test
		'''

		self.browser = web.Browser(None)
		self.challenge = ManualChallenge(
			lambda browser: {'text': 'What is the answer to the question?'},
			lambda data: data['result']
		)

	def test_init(self):
		'''
		Tests that the ManualChallenge initializes correctly
		'''

		self.assertIsInstance(self.challenge, Challenge)

		with self.assertRaises(TypeError):
			ManualChallenge(
				None,
				lambda data: data['result']
			)

		with self.assertRaises(TypeError):
			ManualChallenge(
				lambda browser: {'text': 'What is the answer to the question?'},
				None
			)

	def test_get_data(self):
		'''
		Tests that the ManualChallenge can collect data
		'''

		data = self.challenge.get_data(self.browser)
		self.assertTrue(data['text'])

		with self.assertRaises(TypeError):
			self.challenge.get_data(None)

	def test_submit_data(self):
		'''
		Tests that the ManualChallenge can submit data
		'''

		data = {
			'text': 'What is the answer to the question?',
			'result': 42
		}
		self.assertEqual(self.challenge.submit_data(data), 42)

		with self.assertRaises(TypeError):
			self.challenge.submit_data(None)

	@classmethod
	def tearDownClass(self):
		'''
		Closes the Browser to end the test
		'''

		self.browser.quit()

if __name__ == '__main__':
	unittest.main()



from caboodle import web
import unittest

class BrowserTest(unittest.TestCase):
	'''
	Tests the Browser to verify that it works correctly
	'''

	@classmethod
	def setUpClass(self):
		'''
		Creates a Browser to test
		'''

		self.browser = web.Browser(None)

	def test_init(self):
		'''
		Tests that the Browser initializes correctly
		'''

		self.assertTrue(self.browser.driver)

	def test_proxy(self):
		'''
		Tests that the Browser creates a proxy correctly
		'''

		with self.assertRaises(ValueError):
			web.Browser('invalid_proxy')

		with self.assertRaises(ValueError):
			web.Browser('invalid:proxy')

		browser = web.Browser('127.0.0.1:3128')

		browser.get('https://www.google.com')

		with self.assertRaises(Exception):
			browser.find_element_by_name('q')

		browser.quit()

	def test_get(self):
		'''
		Tests that the Browser can get a URL correctly
		'''

		self.browser.get('https://www.google.com')

		self.assertTrue(self.browser.find_element_by_name('q'))

	def test_actionchains(self):
		'''
		Tests that the Browser can create an ActionChains object
		'''

		self.browser.action().perform()

	def test_timeout(self):
		'''
		Tests that the Browser handles page load timeouts correctly
		'''

		self.browser.set_timeout(0)

		self.browser.get('https://www.google.com')

		with self.assertRaises(ValueError):
			self.browser.set_timeout('abc')

		self.browser.set_timeout(30)

	@classmethod
	def tearDownClass(self):
		'''
		Closes the Browser to end the test
		'''

		self.browser.quit()

if __name__ == '__main__':
	unittest.main()



'''
Utility Functions for Challenges

This module defines useful and frequently used functions for use in Challenges.
Import this module to use them.
'''

from io import BytesIO
from PIL import Image
import base64
import requests

def get_element_image(element, browser):
	'''
	Takes a screenshot and crops out the element

	Args:
		element (WebElement): The element to crop
		browser (Browser): The web browser to use

	Returns:
		A base64 encoded JPEG image
	'''

	result = BytesIO()

	x, y = element.location['x'], element.location['y']
	w, h = element.size['width'], element.size['height']

	screen = BytesIO(base64.b64decode(browser.get_screenshot_as_base64()))
	Image.open(screen).crop((x, y, x + w, y + h)).save(result, format = 'JPEG')

	return base64.b64encode(result.getvalue())

def get_image_src(element):
	'''
	Downloads the source of an image

	Args:
		element (WebElement): The element to download

	Returns:
		A base64 encoded image
	'''

	return base64.b64encode(requests.get(element.get_attribute('src')).content)



'''
ruCaptcha Library

The ruCaptcha library defines functions to interact with ruCaptcha's web API.
See the unit tests for this library for more information.
'''

from io import BytesIO
from PIL import Image
import base64
import json
import requests
import time

def post_image(key, image):
	'''
	Posts an image to be solved

	Args:
		key (str): The API key to use
		image (bytes): The base64 encoded image to post

	Returns:
		The ID of the posted image

	Raises:
		RuntimeError: The post failed
	'''

	data = {
		'method': 'base64',
		'key': key,
		'body': scale_image(image)
	}

	r = requests.post('http://rucaptcha.com/in.php', data = data)

	if r.ok:
		if '|' in r.text:
			response, id = r.text.split('|', 1)

			if response == 'OK':
				return id
			else:
				raise RuntimeError(r.text)
		else:
			raise RuntimeError(r.text)
	else:
		raise RuntimeError('Error %d' % r.status_code)

def post_image_grid(key, image, text, rows, columns):
	'''
	Posts an image grid to be solved

	Args:
		key (str): The API key to use
		image (bytes): The base64 encoded image to post
		text (str): The instructions for the image
		rows (int): The number of rows
		columns (int): The number of columns

	Returns:
		The ID of the posted image

	Raises:
		RuntimeError: The post failed
	'''

	data = {
		'method': 'base64',
		'key': key,
		'recaptcha': 1,
		'textinstructions': text,
		'body': image,
		'row': rows,
		'column': columns
	}

	r = requests.post('http://rucaptcha.com/in.php', data = data)

	if r.ok:
		if '|' in r.text:
			response, id = r.text.split('|', 1)

			if response == 'OK':
				return id
			else:
				raise RuntimeError(r.text)
		else:
			raise RuntimeError(r.text)
	else:
		raise RuntimeError('Error %d' % r.status_code)

def get(key, id):
	'''
	Gets the result of a posted image

	Args:
		key (str): The API key to use
		id (str): The ID of the posted image

	Returns:
		The result of the posted image

	Raises:
		RuntimeError: The get failed
	'''

	params = {
		'key': key,
		'action': 'get',
		'id': id
	}

	while True:
		r = requests.get('http://rucaptcha.com/res.php', params = params)

		if r.ok:
			if '|' in r.text:
				return r.text.split('|', 1)[1]
			elif r.text == 'CAPCHA_NOT_READY':
				pass
			else:
				raise RuntimeError(r.text)
		else:
			raise RuntimeError('Error %d' % r.status_code)

		time.sleep(5)

def invalid(key, id):
	'''
	Reports a posted image as invalid

	Args:
		key (str): The API key to use
		id (str): The ID of the posted image
	'''

	params = {
		'key': key,
		'action': 'reportbad',
		'id': id
	}

	while True:
		r = requests.get('http://rucaptcha.com/res.php', params = params)

		if r.ok:
			break
		else:
			time.sleep(1)

def get_rate(currency, rate = 0.0):
	'''
	Calculates the solving rate in any currency

	Args:
		currency (str): The currency to convert to
		rate (float): A constant solving rate

	Returns:
		The solving rate as a floating point number

	The currency must be a three letter code like USD or EUR. If `rate` is set,
	it will override the rate received from ruCaptcha's API.
	'''

	while rate == 0.0:
		r = requests.get('http://rucaptcha.com/load.php')

		if r.ok:
			rate = r.text.split('\r\n')[3]
			rate = rate.split('>')[1]
			rate = rate.split('<')[0]
			rate = float(rate)

			break
		else:
			time.sleep(1)

	while currency is not 'RUB':
		r = requests.get('http://api.fixer.io/latest?base=RUB')

		if r.ok:
			rates = json.loads(r.text)
			rate *= rates['rates'][currency]

			break
		else:
			time.sleep(1)

	return rate

def scale_image(image):
	'''
	Scales a base64 encoded image to meet ruCaptcha's requirements

	Args:
		image (bytes): The base64 encoded image

	Returns:
		A base64 encoded JPEG image
	'''

	result = BytesIO()
	image = BytesIO(base64.b64decode(image))
	image = Image.open(image)

	w, h = image.size
	scale = 400 / (w + h)

	image = image.resize((int(scale * w), int(scale * h)), Image.ANTIALIAS)
	image.save(result, format = 'JPEG')

	return base64.b64encode(result.getvalue())



from caboodle import web, solve
from caboodle.challenges.manual import ManualChallenge
from caboodle.agents.manual import ManualAgent
import unittest

def foo(data):
	'''
	Solves a Challenge and stores the result

	Args:
		data (dict): The Challenge to solve

	Raises:
		TypeError: The Challenge is not a dictionary
		ValueError: The Challenge does not contain the required data
		RuntimeError: The Challenge could not be solved
	'''

	if 'image' in data:
		raise RuntimeError('The Challenge could not be solved')
	else:
		raise ValueError('The Challenge does not provide an image')

class SolverTest(unittest.TestCase):
	'''
	Tests the Solver to verify that it works correctly
	'''

	@classmethod
	def setUpClass(self):
		'''
		Creates a Solver to test
		'''

		self.browser = web.Browser(None)
		self.solver = solve.Solver(self.browser)

	def test_init(self):
		'''
		Tests that the Solver initializes correctly
		'''

		self.assertTrue(self.solver.browser.driver)
		self.assertIsInstance(self.solver.challenges, list)
		self.assertIsInstance(self.solver.agents, list)

		with self.assertRaises(TypeError):
			solve.Solver(None)

	def test_solve(self):
		'''
		Tests that the Solver can solve a Challenge correctly given there are
		Challenges and Agents to solve those Challenges
		'''

		# Test adding null objects
		with self.assertRaises(TypeError):
			self.solver.add_challenge(None)

		with self.assertRaises(TypeError):
			self.solver.add_agent(None)

		# Test adding valid objects
		self.solver.add_challenge(
			ManualChallenge(
				lambda browser: {},
				lambda data: type(data)
			)
		)
		self.solver.add_challenge(
			ManualChallenge(
				lambda browser: {'text': 'What is the answer to the question?'},
				lambda data: self.assertEqual(data['result'], 42)
			)
		)
		self.assertEqual(len(self.solver.challenges), 2)

		self.solver.add_agent(
			ManualAgent(
				foo,
				lambda data: self.assertTrue(data),
				lambda data: self.assertTrue(data)
			)
		)
		self.solver.add_agent(
			ManualAgent(
				lambda data: data.update({'result': 42}),
				lambda data: self.assertTrue(data),
				lambda data: self.assertTrue(data)
			)
		)
		self.assertEqual(len(self.solver.agents), 2)

		# Test challenge can be solved
		id = self.solver.solve()
		self.solver.set_success(id)
		self.assertEqual(len(self.solver.data), 0)

		with self.assertRaises(KeyError):
			self.solver.set_success('invalid key')

		id = self.solver.solve()
		self.solver.set_fail(id)
		self.assertEqual(len(self.solver.data), 0)

		with self.assertRaises(KeyError):
			self.solver.set_fail('invalid key')

		# Clear all agents
		del self.solver.agents[:]
		self.assertEqual(len(self.solver.agents), 0)

		# Test challenge can't be solved
		with self.assertRaises(RuntimeError):
			self.solver.solve()

	@classmethod
	def tearDownClass(self):
		'''
		Closes the Browser to end the test
		'''

		self.browser.quit()

if __name__ == '__main__':
	unittest.main()



from unittest import mock
import caboodle.libraries.d3cryp7 as lib
import os
import unittest

class FlipFlop(object):
	'''
	Creates a boolean that changes value every time it is used
	'''

	def __init__(self):
		self.state = True

	def __call__(self):
		self.state = not self.state
		return self.state

	def set(self, state):
		'''
		Sets the state of the FlipFlop

		Args:
			state (bool): The state of the FlipFlop
		'''

		self.state = state

class MockResponse(object):
	'''
	Simulates a result from the requests module

	Args:
		text (str): The content returned
		status_code (int): The HTTP status code
	'''

	def __init__(self, text, status_code):
		self.status_code = status_code
		self.ok = status_code == 200
		self.text = text

def flip_flop_get(*args, **kwarg):
	'''
	Replaces the `requests.get()` function by flipping the status code from 404
	to 200 to test the less complex functions `get_rate()`
	'''

	if URL in args[0]:
		if FLIP_FLOP():
			return MockResponse(D3CRYP7_RESPONSE, 200)
		else:
			return MockResponse('', 404)
	elif 'fixer' in args[0]:
		if FLIP_FLOP():
			return MockResponse(FIXER_RESPONSE, 200)
		else:
			return MockResponse('', 404)
	else:
		return MockResponse(None, 404)

URL = 'http://localhost'
HELLO_WORLD = None
CAR = None
TRUCKS = None
BB33 = (
	(  0,   0, 100, 100), (100,   0, 200, 100), (200,   0, 300, 100),
	(  0, 100, 100, 200), (100, 100, 200, 200), (200, 100, 300, 200),
	(  0, 200, 100, 300), (100, 200, 200, 300), (200, 200, 300, 300)
)
BB42 = (
	(  0,   0, 150,  75), (150,   0, 300,  75),
	(  0,  75, 150, 150), (150,  75, 300, 150),
	(  0, 150, 150, 225), (150, 150, 300, 225),
	(  0, 225, 150, 300), (150, 225, 300, 300)
)
FLIP_FLOP = FlipFlop()
D3CRYP7_RESPONSE = None
FIXER_RESPONSE = None

# Locate data files and read them
for root, dirs, files in os.walk(os.getcwd()):
	if 'hello_world' in files:
		HELLO_WORLD = os.path.join(root, 'hello_world')
		HELLO_WORLD = open(HELLO_WORLD, 'rb').read()

	if 'car' in files:
		CAR = os.path.join(root, 'car')
		CAR = open(CAR, 'rb').read()

	if 'commercial_trucks' in files:
		TRUCKS = os.path.join(root, 'commercial_trucks')
		TRUCKS = open(TRUCKS, 'rb').read()

	if 'd3cryp7_response' in files:
		D3CRYP7_RESPONSE = os.path.join(root, 'd3cryp7_response')
		D3CRYP7_RESPONSE = open(D3CRYP7_RESPONSE, 'r').read()
		D3CRYP7_RESPONSE = D3CRYP7_RESPONSE.replace('\n', '\r\n')

	if 'fixer_response' in files:
		FIXER_RESPONSE = os.path.join(root, 'fixer_response')
		FIXER_RESPONSE = open(FIXER_RESPONSE, 'r').read()

class d3cryp7LibraryTest(unittest.TestCase):
	'''
	Tests the d3cryp7 library to verify that it works correctly
	'''

	def test_recognize_image(self):
		'''
		Tests that the function correctly posts an image and returns the result
		'''

		if HELLO_WORLD:
			self.assertEqual(
				lib.recognize_image(URL, HELLO_WORLD),
				'Hello\nWorld'
			)

			with self.assertRaises(RuntimeError):
				lib.recognize_image(URL, None)

			with self.assertRaises(RuntimeError):
				lib.recognize_image('http://google.com', None)
		else:
			self.fail('Could not locate data file')

	def test_tag_image(self):
		'''
		Tests that the function correctly posts an image and returns the result
		'''

		if CAR:
			self.assertIn(
				'car',
				lib.tag_image(URL, CAR)
			)

			with self.assertRaises(RuntimeError):
				lib.tag_image(URL, None)

			with self.assertRaises(RuntimeError):
				lib.tag_image('http://google.com', None)
		else:
			self.fail('Could not locate data file')

	def test_tag_image_grid(self):
		'''
		Tests that the function correctly posts an image and returns the result
		'''

		if TRUCKS:
			for result in lib.tag_image_grid(URL, TRUCKS, 3, 3):
				self.assertEqual(type(result), dict)

			with self.assertRaises(RuntimeError):
				lib.tag_image_grid(URL, None, 1, 1)

			with self.assertRaises(RuntimeError):
				lib.tag_image_grid('http://google.com', None, 1, 1)
		else:
			self.fail('Could not locate data file')

	def test_generate_bbs(self):
		'''
		Tests that the function correctly generates bounding boxes
		'''

		self.assertEqual(lib.generate_bbs(300, 300, 3, 3), BB33)
		self.assertEqual(lib.generate_bbs(300, 300, 2, 4), BB42)

	@mock.patch(
		'caboodle.libraries.d3cryp7.requests.get',
		side_effect = flip_flop_get
	)
	def test_get_cost(self, mock_get):
		'''
		Tests that the function returns the correct result
		'''

		if D3CRYP7_RESPONSE and FIXER_RESPONSE:
			data = lib.get_rate(URL, 'EUR')
			self.assertIn('recognize', data)
			self.assertIn('tag', data)

			FLIP_FLOP.set(True)
		else:
			self.fail('Could not locate data file')

if __name__ == '__main__':
	unittest.main()



'''
Solve Media CAPTCHA Challenges

This module is an implementation of the Challenge specification and collects
data to solve Solve Media CAPTCHAs. To use these Challenges, create a new
instance of them and call the `get_data()` function. Then, process the data
using an Agent and submit it by calling the `submit_data()` function. See the
unit tests for this module for more information.
'''

from caboodle.challenges.spec import Challenge
import caboodle.challenges.util as util

class SolveMediaTextChallenge(Challenge):
	'''
	A Challenge for Solve Media Text CAPTCHAs

	A text CAPTCHA is the traditional type of CAPTCHA with obfuscated squiggly
	text. This Challenge will locate it if it exists and add a base64 encoded
	image of the CAPTCHA to the dictionary with the key 'image'. In addition to
	the CAPTCHA, the form to enter the result and the reload button to get a new
	CAPTCHA are added to the dictionary with the keys 'form' and 'reload'
	respectively.
	'''

	def __init__(self):
		super().__init__()

	def get_data(self, browser):
		'''
		Collects data needed to solve the Challenge

		Args:
			browser (Browser): The web browser to use

		Returns:
			A dictionary of collected data

		Raises:
			TypeError: The browser is not of type Browser
		'''

		super().get_data(browser)

		try:
			data = {}

			# Get elements
			data['image'] = browser.find_element_by_id('adcopy-puzzle-image-image')
			data['form'] = browser.find_element_by_id('adcopy_response')
			data['reload'] = browser.find_element_by_id('adcopy-link-refresh')

			# Preprocess elements
			data['image'] = util.get_element_image(data['image'], browser)

			return data
		except:
			return None

	def submit_data(self, data):
		'''
		Submits the processed data and solves the Challenge

		Args:
			data (dict): The Challenge to submit

		Raises:
			TypeError: The data is not a dictionary
		'''

		super().submit_data(data)

		data['form'].send_keys(data['result'])



from io import BytesIO
from PIL import Image
from unittest import mock
import base64
import caboodle.libraries.rucaptcha as lib
import os
import unittest

class Counter(object):
	'''
	Creates an integer counter that increments the value every time it is used
	'''

	def __init__(self):
		self.state = 0

	def __call__(self):
		self.state += 1
		return self.state

	def set(self, state):
		'''
		Sets the state of the Counter

		Args:
			state (int): The state of the Counter
		'''

		self.state = state

class FlipFlop(object):
	'''
	Creates a boolean that changes value every time it is used
	'''

	def __init__(self):
		self.state = True

	def __call__(self):
		self.state = not self.state
		return self.state

	def set(self, state):
		'''
		Sets the state of the FlipFlop

		Args:
			state (bool): The state of the FlipFlop
		'''

		self.state = state

class MockResponse(object):
	'''
	Simulates a result from the requests module

	Args:
		text (str): The content returned
		status_code (int): The HTTP status code
	'''

	def __init__(self, text, status_code):
		self.status_code = status_code
		self.ok = status_code == 200
		self.text = text

def counter_post(*args, **kwags):
	'''
	Replaces the `requests.post()` function by stepping through a series of
	states to test the more complex `post()` function
	'''

	state = COUNTER()
	if state == 1:
		return MockResponse('', 404)
	elif state == 2:
		return MockResponse('ERROR_NO_SLOT_AVAILABLE', 200)
	elif state == 3:
		return MockResponse('THIS_SHOULD|NEVER_HAPPEN', 200)
	elif state == 4:
		return MockResponse('OK|42', 200)
	else:
		return MockResponse(None, 404)

def counter_get_image(*args, **kwags):
	'''
	Replaces the `requests.get()` function by stepping through a series of
	states to test the more complex `get()` function
	'''

	state = COUNTER()
	if state == 1:
		return MockResponse('', 404)
	elif state == 2:
		return MockResponse('ERROR_KEY_DOES_NOT_EXIST', 200)
	elif state == 3:
		return MockResponse('CAPCHA_NOT_READY', 200)
	elif state == 4:
		return MockResponse('OK|Python', 200)
	else:
		return MockResponse(None, 404)

def counter_get_image_grid(*args, **kwags):
	'''
	Replaces the `requests.get()` function by stepping through a series of
	states to test the more complex `get()` function
	'''

	state = COUNTER()
	if state == 1:
		return MockResponse('', 404)
	elif state == 2:
		return MockResponse('ERROR_KEY_DOES_NOT_EXIST', 200)
	elif state == 3:
		return MockResponse('CAPCHA_NOT_READY', 200)
	elif state == 4:
		return MockResponse('OK|click:1/4/5', 200)
	else:
		return MockResponse(None, 404)

def flip_flop_get(*args, **kwarg):
	'''
	Replaces the `requests.get()` function by flipping the status code from 404
	to 200 to test the less complex functions `invalid()` and `get_rate()`
	'''

	if 'rucaptcha' in args[0]:
		if FLIP_FLOP():
			if 'params' in kwarg:
				return MockResponse('', 200)
			else:
				return MockResponse(RUCAPTCHA_RESPONSE, 200)
		else:
			return MockResponse('', 404)
	elif 'fixer' in args[0]:
		if FLIP_FLOP():
			return MockResponse(FIXER_RESPONSE, 200)
		else:
			return MockResponse('', 404)
	else:
		return MockResponse(None, 404)

PYTHON = None
TRUCKS = None
RUCAPTCHA_RESPONSE = None
FIXER_RESPONSE = None
FLIP_FLOP = FlipFlop()
COUNTER = Counter()

# Locate data files and read them
for root, dirs, files in os.walk(os.getcwd()):
	if 'python' in files:
		PYTHON = os.path.join(root, 'python')
		PYTHON = open(PYTHON, 'rb').read()

	if 'commercial_trucks' in files:
		TRUCKS = os.path.join(root, 'commercial_trucks')
		TRUCKS = open(TRUCKS, 'rb').read()

	if 'rucaptcha_response' in files:
		RUCAPTCHA_RESPONSE = os.path.join(root, 'rucaptcha_response')
		RUCAPTCHA_RESPONSE = open(RUCAPTCHA_RESPONSE, 'r').read()
		RUCAPTCHA_RESPONSE = RUCAPTCHA_RESPONSE.replace('\n', '\r\n')

	if 'fixer_response' in files:
		FIXER_RESPONSE = os.path.join(root, 'fixer_response')
		FIXER_RESPONSE = open(FIXER_RESPONSE, 'r').read()

class RucaptchaLibraryTest(unittest.TestCase):
	'''
	Tests the ruCaptcha library to verify that it works correctly
	'''

	@mock.patch(
		'caboodle.libraries.rucaptcha.requests.post',
		side_effect = counter_post
	)
	def test_post_image(self, mock_post):
		'''
		Tests that the function correctly posts an image and returns the ID
		'''

		if PYTHON:
			with self.assertRaises(RuntimeError):
				lib.post_image('API KEY', PYTHON)

			with self.assertRaises(RuntimeError):
				lib.post_image('API KEY', PYTHON)

			with self.assertRaises(RuntimeError):
				lib.post_image('API KEY', PYTHON)

			self.assertEqual(lib.post_image('API KEY', PYTHON), '42')
			COUNTER.set(0)
		else:
			self.fail('Could not locate data file')

	@mock.patch(
		'caboodle.libraries.rucaptcha.requests.post',
		side_effect = counter_post
	)
	def test_post_image_grid(self, mock_post):
		'''
		Tests that the function correctly posts an image grid and returns the ID
		'''

		if TRUCKS:
			with self.assertRaises(RuntimeError):
				lib.post_image_grid(
					'API KEY',
					TRUCKS,
					'Select all images with commercial trucks',
					3,
					3
				)

			with self.assertRaises(RuntimeError):
				lib.post_image_grid(
					'API KEY',
					TRUCKS,
					'Select all images with commercial trucks',
					3,
					3
				)

			with self.assertRaises(RuntimeError):
				lib.post_image_grid(
					'API KEY',
					TRUCKS,
					'Select all images with commercial trucks',
					3,
					3
				)

			self.assertEqual(
				lib.post_image_grid(
					'API KEY',
					TRUCKS,
					'Select all images with commercial trucks',
					3,
					3
				),
				'42'
			)
			COUNTER.set(0)
		else:
			self.fail('Could not locate data file')

	@mock.patch(
		'caboodle.libraries.rucaptcha.requests.get',
		side_effect = counter_get_image
	)
	def test_get_image(self, mock_get):
		'''
		Tests that the function returns the result of the posted image
		'''

		with self.assertRaises(RuntimeError):
			lib.get('API KEY', 42)

		with self.assertRaises(RuntimeError):
			lib.get('API KEY', 42)

		self.assertEqual(lib.get('API KEY', 42), 'Python')
		COUNTER.set(0)

	@mock.patch(
		'caboodle.libraries.rucaptcha.requests.get',
		side_effect = counter_get_image_grid
	)
	def test_get_image_grid(self, mock_get):
		'''
		Tests that the function returns the result of the posted image grid
		'''

		with self.assertRaises(RuntimeError):
			lib.get('API KEY', 42)

		with self.assertRaises(RuntimeError):
			lib.get('API KEY', 42)

		self.assertEqual(lib.get('API KEY', 42), 'click:1/4/5')
		COUNTER.set(0)

	@mock.patch(
		'caboodle.libraries.rucaptcha.requests.get',
		side_effect = flip_flop_get
	)
	def test_invalid(self, mock_get):
		'''
		Tests that the function works correctly and does not raise any
		exceptions
		'''

		lib.invalid('API KEY', 42)
		FLIP_FLOP.set(True)

	@mock.patch(
		'caboodle.libraries.rucaptcha.requests.get',
		side_effect = flip_flop_get
	)
	def test_get_rate(self, mock_get):
		'''
		Tests that the function returns the correct result
		'''

		if RUCAPTCHA_RESPONSE and FIXER_RESPONSE:
			self.assertEqual(int(lib.get_rate('USD') * 10000000000), 7448445)
			FLIP_FLOP.set(True)
		else:
			self.fail('Could not locate data file')

	def test_scale_image(self):
		'''
		Tests that the function can scale an image while maintaining the aspect
		ratio to within 0.1% of the original
		'''

		ratio = 290 / 82
		ratio_plus = ratio + ratio * 0.001
		ratio_minus = ratio - ratio * 0.001

		if PYTHON:
			image = lib.scale_image(PYTHON)
			image = BytesIO(base64.b64decode(image))
			image = Image.open(image)

			w, h = image.size
			actual = w / h

			if actual < ratio_minus and actual > ratio_plus:
				difference = actual - ratio
				average = (actual + ratio) / 2

				# Absolute value
				if difference < 0.0:
					difference -= 2 * difference

				percent_diff = (difference / average) * 100

				self.fail('The aspect ratio was %f% different' % percent_diff)
		else:
			self.fail('Could not locate data file')

if __name__ == '__main__':
	unittest.main()



'''
A local Challenge solving Agent

This module is an implementation of the Agent specification and solves a
Challenge using the `input()` function. To use this Agent, create a new instance
of it and call the `solve()` function. See the unit tests for this module for
more information.
'''

from caboodle.agents.spec import Agent

class LocalAgent(Agent):
	'''
	An Agent to solve Challenges with input from the user

	This Agent uses the `input()` function and thus will block the execution of
	your application until it receives input. The result is appended to the
	dictionary with the key 'result' as per the guidelines. Because this Agent
	relies on user input, it has a cost of zero.
	'''

	def __init__(self):
		super().__init__()

	def solve(self, data):
		'''
		Solves a Challenge and stores the result

		Args:
			data (dict): The Challenge to solve

		Raises:
			TypeError: The Challenge is not a dictionary
		'''

		super().solve(data)

		data['result'] = input('Please solve the challenge: ')

	def get_cost(self):
		'''
		Returns the cost of using this Agent

		Returns:
			The cost as a floating point number
		'''

		super().get_cost()

		return self._cost

	def success(self, data):
		'''
		Performs actions for a successful Challenge

		Args:
			data (dict): The successful Challenge
		'''

		super().success(data)

	def fail(self, data):
		'''
		Performs actions for a failed Challenge

		Args:
			data (dict): The failed Challenge
		'''

		super().fail(data)



'''
Challenge Solving Utilities

This module uses Agents to solve Challenges. Both are modular and any number of
them can be added. A Solver works by running every loaded Challenge and checking
if it returned any data. If it did, then every Solver is ran from least cost to
greatest until a result is found or a RuntimeError is raised indicating that the
Challenge could not be solved. Because of the way the Solver works, it is
imperative that every Challenge and Agent operate according to their
specifications. See the unit tests for this module for more information.
'''

from caboodle.agents.spec import Agent
from caboodle.challenges.spec import Challenge
from caboodle.web import Browser
from time import time

class Solver(object):
	'''
	Solves challenges in a Browser

	Args:
		browser (Browser): The web browser to use

	Raises:
		TypeError: The browser is not of type Browser
	'''

	def __init__(self, browser):
		if type(browser) is Browser:
			self.browser = browser
		else:
			raise TypeError('The browser must be of type Browser')

		self.challenges = []
		self.agents = []
		self.data = {}

	def solve(self):
		'''
		Solves a Challenge

		Returns:
			The ID of the Challenge

		Raises:
			RuntimeError: The Challenge could not be solved
		'''

		self.agents.sort()

		for challenge in self.challenges:
			self.browser.switch_to_default_content()

			data = challenge.get_data(self.browser)

			if data == 'solved':
				return None

			if not data:
				continue

			for agent in self.agents:
				try:
					agent.solve(data)
					challenge.submit_data(data)
					data['agent'] = agent
					id = hex(int(time() * 10000000))[2:]
					self.data[id] = data

					self.browser.switch_to_default_content()

					return id
				except ValueError or RuntimeError:
					continue

		self.browser.switch_to_default_content()

		raise RuntimeError('The Challenge could not be solved')

	def add_challenge(self, challenge):
		'''
		Adds a Challenge to the list of challenge types

		Args:
			challenge (Challenge): The Challenge to add

		Raises:
			TypeError: The challenge is not of type Challenge
		'''

		if isinstance(challenge, Challenge):
			self.challenges.append(challenge)
		else:
			raise TypeError('The challenge must be of type Challenge')

	def add_agent(self, agent):
		'''
		Adds an Agent to the list of available solving agents

		Args:
			agent (Agent): The Agent to add

		Raises:
			TypeError: The agent is not of type Agent
		'''

		if isinstance(agent, Agent):
			self.agents.append(agent)
		else:
			raise TypeError('The agent must be of type Agent')

	def set_success(self, id):
		'''
		Sets a Challenge as successful and handles it appropriately before
		removing it from the list

		Args:
			id (str): The ID of the Challenge

		Raises:
			KeyError: The ID was not found
		'''

		if id:
			self.data[id]['agent'].success(self.data[id])
			del self.data[id]

	def set_fail(self, id):
		'''
		Sets a Challenge as failed and handles it appropriately before removing
		it from the list

		Args:
			id (str): The ID of the Challenge

		Raises:
			KeyError: The ID was not found
		'''

		if id:
			self.data[id]['agent'].fail(self.data[id])
			del self.data[id]



from caboodle import web
from caboodle.challenges.recaptcha import RecaptchaV2Challenge
from caboodle.challenges.spec import Challenge
import unittest

class RecaptchaV2ChallengeTest(unittest.TestCase):
	'''
	Tests the RecaptchaV2Challenge to verify that it works correctly
	'''

	@classmethod
	def setUpClass(self):
		'''
		Creates a Challenge to test
		'''

		self.browser = web.Browser(None)
		self.challenge = RecaptchaV2Challenge()

	def test_init(self):
		'''
		Tests that the RecaptchaV2Challenge initializes correctly
		'''

		self.assertIsInstance(self.challenge, Challenge)

	def test_get_data(self):
		'''
		Tests that the RecaptchaV2Challenge can collect data
		'''

		self.assertFalse(self.challenge.get_data(self.browser))

		self.browser.get('https://www.google.com/recaptcha/api2/demo')

		data = self.challenge.get_data(self.browser)
		self.assertTrue(data['image'] or data == 'solved')
		self.assertTrue(data['tiles'] or data == 'solved')
		self.assertTrue(data['verify'] or data == 'solved')
		self.assertTrue(data['text'] or data == 'solved')
		self.assertTrue(data['tag'] or data == 'solved')
		self.assertTrue(data['reload'] or data == 'solved')
		self.assertTrue(data['rows'] or data == 'solved')
		self.assertTrue(data['columns'] or data == 'solved')

		with self.assertRaises(TypeError):
			self.challenge.get_data(None)

	def test_submit_data(self):
		'''
		Tests that the RecaptchaV2Challenge can submit data
		'''

		self.browser.get('https://www.google.com/recaptcha/api2/demo')

		data = self.challenge.get_data(self.browser)
		data['result'] = (0, 1, 2, 3, 4, 5, 6, 7, 8, 9)

		self.challenge.submit_data(data)

	@classmethod
	def tearDownClass(self):
		'''
		Closes the Browser to end the test
		'''

		self.browser.quit()

if __name__ == '__main__':
	unittest.main()



from caboodle.agents.manual import ManualAgent
import unittest

def foo(data):
	'''
	Solves a Challenge and stores the result

	Args:
		data (dict): The Challenge to solve

	Raises:
		TypeError: The Challenge is not a dictionary
		ValueError: The Challenge does not contain the required data
		RuntimeError: The Challenge could not be solved
	'''

	if data:
		data['result'] = 42
	else:
		raise TypeError('The Challenge must be a dictionary')

class ManualAgentTest(unittest.TestCase):
	'''
	Tests the ManualAgent to verify that it works correctly
	'''

	@classmethod
	def setUpClass(self):
		'''
		Creates a ManualAgent to test
		'''

		self.agent = ManualAgent(
			foo,
			lambda data: True,
			lambda data: False
		)

	def test_init(self):
		'''
		Tests that the ManualAgent initializes correctly
		'''

		self.assertEqual(self.agent._cost, 0.0)

		with self.assertRaises(TypeError):
			ManualAgent(
				None,
				lambda data: type(data),
				lambda data: type(data)
			)

		with self.assertRaises(TypeError):
			ManualAgent(
				lambda data: type(data),
				None,
				lambda data: type(data)
			)

		with self.assertRaises(TypeError):
			ManualAgent(
				lambda data: type(data),
				lambda data: type(data),
				None
			)

	def test_solve(self):
		'''
		Tests that the ManualAgent solves a Challenge correctly
		'''

		data = {'text': 'What is the answer to the question?'}

		self.agent.solve(data)
		self.assertEqual(data['result'], 42)

		with self.assertRaises(TypeError):
			self.agent.solve(None)

	def test_cost(self):
		'''
		Tests that the ManualAgent returns the correct cost
		'''

		self.assertEqual(self.agent.get_cost(), 0.0)

	def test_comparable(self):
		'''
		Tests that a ManualAgent can be compared to another Agent
		'''

		self.assertFalse(self.agent < self.agent)

		with self.assertRaises(TypeError):
			self.agent < None

	def test_success(self):
		'''
		Tests that a ManualAgent can handle a successful Challenge correctly
		'''

		data = {
			'text': 'What is the answer to the question?',
			'result': 42
		}

		self.assertTrue(self.agent.success(data))

	def test_fail(self):
		'''
		Tests that a ManualAgent can handle a failed Challenge correctly
		'''

		data = {
			'text': 'What is the answer to the question?',
			'result': 3.14
		}

		self.assertFalse(self.agent.fail(data))

if __name__ == '__main__':
	unittest.main()



'''
d3cryp7 Library

The d3cryp7 library defines functions to interact with d3cryp7's web API.
See the unit tests for this library for more information.
'''

from io import BytesIO
from PIL import Image
import base64
import json
import requests
import time

def recognize_image(url, image):
	'''
	Posts an image to be recognized

	Args:
		url (string): The URL of the d3cryp7 API
		image (bytes): The base64 encoded image to post

	Returns:
		A string containing the text in the image

	Raises:
		RuntimeError: The post failed
	'''

	data = {
		'image': image
	}

	r = requests.post(url + '/api/recognize', data = data)

	if r.ok:
		response = r.json()

		if response['result'] and len(response['result']) > 0:
			return response['result']
		else:
			raise RuntimeError('No result')
	else:
		raise RuntimeError('Error %d' % r.status_code)

def tag_image(url, image):
	'''
	Posts an image to be tagged

	Args:
		url (string): The URL of the d3cryp7 API
		image (bytes): The base64 encoded image to post

	Returns:
		A dictionary of tags and their probabilities

	Raises:
		RuntimeError: The post failed
	'''

	data = {
		'image': image
	}

	r = requests.post(url + '/api/tag', data = data)

	if r.ok:
		response = r.json()

		if response['result'] and len(response['result']) > 0:
			return response['result']
		else:
			raise RuntimeError('No result')
	else:
		raise RuntimeError('Error %d' % r.status_code)

def tag_image_grid(url, image, col, row):
	'''
	Posts an image grid to be tagged

	Args:
		url (string): The URL of the d3cryp7 API
		image (bytes): The base64 encoded image to post
		col (int): The number of columns in the image
		row (int): The number of rows in the image

	Returns:
		A tuple of dictionaries of tags and their probabilities

	Raises:
		RuntimeError: The post failed
	'''

	if not image:
		raise RuntimeError('Image cannot be null')

	result = ()
	im = Image.open(BytesIO(base64.b64decode(image)))
	w, h = im.size
	bbs = generate_bbs(w, h, col, row)

	for bb in bbs:
		im2 = BytesIO()
		temp = im.crop(bb)
		temp.save(im2, format = 'JPEG')
		im2 = base64.b64encode(im2.getvalue())
		result += (tag_image(url, im2),)

	return result

def generate_bbs(w, h, col, row):
	'''
	Generates a tuple of bounding boxes to crop a grid of images

	Args:
		w (int): The width of an image
		h (int): The height of an image
		col (int): The number of columns in an image
		row (int): The number of rows in an image

	Returns:
		A tuple of tuples that contain 4 integers
	'''

	cell_w = w / col
	cell_h = h / row

	return tuple(
		(
			int(cell_w * j), int(cell_h * i),
			int(cell_w * j + cell_w), int(cell_h * i + cell_h)
		)
		for i in range(row)
		for j in range(col)
	)

def get_rate(url, currency):
	'''
	Calculates the solving rate in any currency

	Args:
		url (string): The URL of the d3cryp7 API
		currency (str): The currency to convert to

	Returns:
		A dictionary of solving rates as floating point numbers

	The currency must be a three letter code like USD or EUR.
	'''

	rate = None

	while not rate:
		r = requests.get(url + '/api/cost')

		if r.ok:
			rate = json.loads(r.text)

			break
		else:
			time.sleep(1)

	while currency is not 'USD':
		r = requests.get('http://api.fixer.io/latest?base=USD')

		if r.ok:
			rates = json.loads(r.text)

			for key in rate.keys():
				rate[key] *= rates['rates'][currency]

			break
		else:
			time.sleep(1)

	return rate



'''
A manually defined Challenge

This module is an implementation of the Challenge specification and collects
data using an external function defined by the user. To use this Challenge,
create a new instance of it with the desired functions and call the `get_data()`
function. Then, process the data using an Agent and submit it by calling the
`submit_data()` function. See the unit tests for this module for more
information.
'''

from caboodle.challenges.spec import Challenge

class ManualChallenge(Challenge):
	'''
	A Challenge that uses external functions

	Args:
		get_function (function): The function to collect data
		submit_function (function): The function to submit data

	Raises:
		TypeError: The arguments are not functions

	As a general guideline, processed data should contain a key named 'result'
	and so the `submit_function` may depend on that when submitting. In
	addition, collected data should be organized logically. For example, if the
	Challenge requires an image to be solved, it should be base64 encoded and
	appended to the dictionary with the key 'image'.
	'''

	def __init__(self, get_function, submit_function):
		super().__init__()

		if callable(get_function):
			self.get_function = get_function
		else:
			raise TypeError('The get function must be a function')

		if callable(submit_function):
			self.submit_function = submit_function
		else:
			raise TypeError('The submit function must be a function')

	def get_data(self, browser):
		'''
		Collects data needed to solve the Challenge

		Args:
			browser (Browser): The web browser to use

		Returns:
			A dictionary of collected data

		Raises:
			TypeError: The browser is not of type Browser
		'''

		super().get_data(browser)

		return self.get_function(browser)

	def submit_data(self, data):
		'''
		Submits the processed data and solves the Challenge

		Args:
			data (dict): The Challenge to submit

		Raises:
			TypeError: The data is not a dictionary
		'''

		super().submit_data(data)

		return self.submit_function(data)



'''
Web Interfaces

This module defines objects for interacting with the World Wide Web. See the
documentation for each object and their respective unit tests for more
information.
'''

from selenium import webdriver
from selenium.common.exceptions import TimeoutException
from selenium.webdriver.common.proxy import *
from selenium.webdriver.common.action_chains import ActionChains

class Browser(object):
	'''
	A web browser

	Args:
		proxy (str): The proxy to use

	Raises:
		ValueError: The proxy is invalid

	Browser is a wrapper for the web browser automation tool Selenium. To
	simplify things and to avoid linking to external binaries, only Firefox
	is supported. Firefox natively supports Selenium without any configuration.

	A proxy may be used and must be in the form of `host:port`. The host
	can be a fully qualified domain name (FQDN) or an IP address. The port
	must be a valid integer. You may also set the proxy to None for no proxy.

	Example:

		127.0.0.1:3128
		localhost:3128
		None
	'''

	def __init__(self, proxy = None):
			if proxy:
				if proxy.count(':') is not 1:
					raise ValueError('Proxy %s is invalid' % proxy)

				port = proxy.split(':')[1]
				try:
					int(port)
				except:
					raise ValueError('Proxy port %s is invalid' % port)

				proxy = Proxy({
					'proxyType': ProxyType.MANUAL,
					'httpProxy': proxy,
					'ftpProxy': proxy,
					'sslProxy': proxy
				})

				self.driver = webdriver.Firefox(proxy = proxy)
			else:
				self.driver = webdriver.Firefox()

	def __getattr__(self, attr):
		return self.driver.__getattribute__(attr)

	def get(self, url):
		'''
		Requests a URL in the browser

		Args:
			url (str): The URL to request

		If the page load time exceeds the default timeout limit, it will
		automatically be handled by forcing the Browser to stop loading.
		You can set the default timeout with the `set_timeout()` function.
		'''

		try:
			self.driver.get(url)
		except TimeoutException:
			self.driver.execute_script('window.stop();')

	def action(self):
		'''
		Creates an ActionChains object to string together operations

		Returns:
			An ActionChains object

		An action chain is useful for doing complex actions like hover over
		and drag and drop. This function returns an ActionChains object with
		the Browser passed as an argument and can be used just like the
		original found here:

		http://selenium-python.readthedocs.io/api.html
		'''

		return ActionChains(self.driver)

	def set_timeout(self, time):
		'''
		Changes the length of time a page is allowed to load before it stops

		Args:
			time (int): The page load time in seconds

		Raises:
			ValueError: The page load time is invalid
		'''

		if type(time) is int:
			self.driver.set_page_load_timeout(time)
		else:
			raise ValueError('%s is an invalid time' % time)



'''
Specifications for Challenge objects

This module defines abstract classes with compounding levels of functionality.
Every Challenge should have a `get_data()` and `submit_data()` function. These
specifications can not be instantiated and can only be used by a subclass that
implements all of the required functions. Be sure, however, to explicitly call
the `super()` function from each subclass and from each defined function in
that subclass to gain the functionality of it's parent. See the unit tests for
this module for more information.
'''

from abc import ABCMeta, abstractmethod
from caboodle.web import Browser

class Challenge(object, metaclass = ABCMeta):
	'''
	An object that represents a challenge

	A Challenge collects data and packages it into a dictionary to be processed.
	Once that data has been processed, it can then submit the result in the
	required way. As a general guideline, processed data should contain a key
	named 'result' and so a Challenge may depend on that when submitting. In
	addition, collected data should be organized logically. For example, if the
	Challenge requires an image to be solved, it should be base64 encoded and
	appended to the dictionary with the key 'image'.
	'''

	def __init__(self):
		pass

	@abstractmethod
	def get_data(self, browser):
		'''
		Collects data needed to solve the Challenge

		Args:
			browser (Browser): The web browser to use

		Returns:
			A dictionary of collected data

		Raises:
			TypeError: The browser is not of type Browser
		'''

		if type(browser) is not Browser:
			raise TypeError('The browser must be of type Browser')

	@abstractmethod
	def submit_data(self, data):
		'''
		Submits the processed data and solves the Challenge

		Args:
			data (dict): The Challenge to submit

		Raises:
			TypeError: The data is not a dictionary
		'''

		if type(data) is not dict:
			raise TypeError('The data must be a dictionary')



from caboodle.agents.spec import Agent
import unittest

class GoodAgent(Agent):
	'''
	A complete child of the Agent specification
	'''

	def __init__(self):
		super().__init__()

		self._cost = 3.14

	def solve(self, data):
		'''
		Solves a Challenge and stores the result

		Args:
			data (dict): The Challenge to solve

		Raises:
			TypeError: The Challenge is not a dictionary
		'''

		super().solve(data)

		data['result'] = 42

	def get_cost(self):
		'''
		Returns the cost of using this Agent

		Returns:
			The cost as a floating point number
		'''

		super().get_cost()

		return self._cost

	def success(self, data):
		'''
		Performs actions for a successful Challenge

		Args:
			data (dict): The successful Challenge
		'''

		super().success(data)

		return True

	def fail(self, data):
		'''
		Performs actions for a failed Challenge

		Args:
			data (dict): The failed Challenge
		'''

		super().fail(data)

		return False

class BadAgent(Agent):
	'''
	An incomplete child of the Agent specification
	'''

	def __init__(self):
		super().__init__()

class AgentTest(unittest.TestCase):
	'''
	Tests the Agent specification to verify that it works correctly
	'''

	def test_abstraction(self):
		'''
		Tests that an incomplete child of the Agent specification will
		raise a TypeError
		'''

		with self.assertRaises(TypeError):
			BadAgent()

	def test_init(self):
		'''
		Tests that the Agent specification initializes correctly
		'''

		self.assertIsInstance(GoodAgent(), Agent)
		self.assertEqual(GoodAgent()._cost, 3.14)

	def test_solve(self):
		'''
		Tests that the Agent specification solves a Challenge correctly
		'''

		data = {'text': 'What is the answer to the question?'}

		GoodAgent().solve(data)
		self.assertEqual(data['result'], 42)

		with self.assertRaises(TypeError):
			GoodAgent().solve(None)

	def test_cost(self):
		'''
		Tests that the Agent specification returns the correct cost
		'''

		self.assertEqual(GoodAgent().get_cost(), 3.14)

	def test_comparable(self):
		'''
		Tests that an Agent can be compared to another Agent
		'''

		self.assertFalse(GoodAgent() < GoodAgent())

		with self.assertRaises(TypeError):
			GoodAgent() < None

	def test_success(self):
		'''
		Tests that an Agent can handle a successful Challenge correctly
		'''

		data = {
			'text': 'What is the answer to the question?',
			'result': 42
		}

		self.assertTrue(GoodAgent().success(data))

	def test_fail(self):
		'''
		Tests that an Agent can handle a failed Challenge correctly
		'''

		data = {
			'text': 'What is the answer to the question?',
			'result': 3.14
		}

		self.assertFalse(GoodAgent().fail(data))

if __name__ == '__main__':
	unittest.main()



from setuptools import setup, find_packages

def format(input, start = 0):
	result = ''
	indent = False
	count = 0

	with open(input, 'r') as file:
		for line in file:
			if count > start:
				if line[:1] == '\t' and not indent:
					indent = True
					result += '::\n\n'

				if line[:1].isalnum() and indent:
					indent = False

				result += line.replace('> ', '\t')
			count += 1

	return result

blurb = ('Caboodle is a Python module for web browsing, web scraping or web '
	'automation developed to provide an all-in-one (kit and caboodle) utility '
	'for anything the web has to offer.\n'
)

setup(
	name = 'Caboodle',
	version = '1.0.0a3',
	author = 'Justin Willis',
	author_email = 'sirJustin.Willis@gmail.com',
	packages = find_packages(),
	url = 'https://bitbucket.org/bkvaluemeal/caboodle',
	license = 'ISC License',
	description =
		'A Python module for web browsing, web scraping or web automation',
	long_description = blurb + format('README.md', 3),
	classifiers = [
		'Development Status :: 3 - Alpha',
		'Environment :: Console',
		'Intended Audience :: Developers',
		'License :: OSI Approved :: ISC License (ISCL)',
		'Operating System :: OS Independent',
		'Programming Language :: Python :: 2',
		'Programming Language :: Python :: 2.6',
		'Programming Language :: Python :: 2.7',
		'Programming Language :: Python :: 3',
		'Topic :: Other/Nonlisted Topic'
	],
	keywords = 'web browsing scraping automation',
	install_requires = [
		'Pillow',
		'Requests',
		'Selenium'
	]
)



from caboodle import web
from caboodle.challenges.solve_media import SolveMediaTextChallenge
from caboodle.challenges.spec import Challenge
import unittest

class SolveMediaTextChallengeTest(unittest.TestCase):
	'''
	Tests the SolveMediaTextChallenge to verify that it works correctly
	'''

	@classmethod
	def setUpClass(self):
		'''
		Creates a Challenge to test
		'''

		self.browser = web.Browser(None)
		self.challenge = SolveMediaTextChallenge()

	def test_init(self):
		'''
		Tests that the SolveMediaTextChallenge initializes correctly
		'''

		self.assertIsInstance(self.challenge, Challenge)

	def test_get_data(self):
		'''
		Tests that the SolveMediaTextChallenge can collect data
		'''

		self.assertFalse(self.challenge.get_data(self.browser))

		self.browser.get(
			'http://wsnippets.com/demo/solve-media-captcha-php/index.php'
		)

		data = self.challenge.get_data(self.browser)
		self.assertTrue(data['image'])
		self.assertTrue(data['form'])
		self.assertTrue(data['reload'])

		with self.assertRaises(TypeError):
			self.challenge.get_data(None)

	def test_submit_data(self):
		'''
		Tests that the SolveMediaTextChallenge can submit data
		'''

		self.browser.get(
			'http://wsnippets.com/demo/solve-media-captcha-php/index.php'
		)

		data = self.challenge.get_data(self.browser)
		data['result'] = '42'

		self.challenge.submit_data(data)

		self.assertEqual(data['form'].get_attribute('value'), '42')

	@classmethod
	def tearDownClass(self):
		'''
		Closes the Browser to end the test
		'''

		self.browser.quit()

if __name__ == '__main__':
	unittest.main()



from caboodle import web
import caboodle.challenges.util as util
import os
import unittest

DEBIAN = None
WIKIPEDIA = None

# Locate data file and read it
for root, dirs, files in os.walk(os.getcwd()):
	if 'debian' in files:
		DEBIAN = os.path.join(root, 'debian')
		DEBIAN = open(DEBIAN, 'rb').read()

	if 'wikipedia' in files:
		WIKIPEDIA = os.path.join(root, 'wikipedia')
		WIKIPEDIA = open(WIKIPEDIA, 'rb').read()

class UtilTest(unittest.TestCase):
	'''
	Tests the Challenge utility functions to verify they work correctly
	'''

	@classmethod
	def setUpClass(self):
		'''
		Creates a Browser to use
		'''

		self.browser = web.Browser(None)

	def test_get_element_image(self):
		'''
		Tests that the function returns the correct image
		'''

		if WIKIPEDIA:
			self.browser.get('https://www.wikipedia.org')

			image = self.browser.find_element_by_tag_name('img')
			image = util.get_element_image(image, self.browser)

			self.assertEqual(image, WIKIPEDIA)
		else:
			self.fail('Could not locate data file')

	def test_get_image_src(self):
		'''
		Tests that the function returns the correct image
		'''

		if DEBIAN:
			self.browser.get('https://www.debian.org')

			image = self.browser.find_element_by_tag_name('img')
			image = util.get_image_src(image)

			self.assertEqual(image, DEBIAN)
		else:
			self.fail('Could not locate data file')

	@classmethod
	def tearDownClass(self):
		'''
		Closes the Browser to end the test
		'''

		self.browser.quit()

if __name__ == '__main__':
	unittest.main()



from caboodle.agents.d3cryp7 import d3cryp7TagImageGridAgent
import os
import unittest

TRUCKS = None

# Locate data file and read it
for root, dirs, files in os.walk(os.getcwd()):
	if 'commercial_trucks' in files:
		TRUCKS = os.path.join(root, 'commercial_trucks')
		TRUCKS = open(TRUCKS, 'rb').read()

class d3cryp7TagImageGridAgentTest(unittest.TestCase):
	'''
	Tests the d3cryp7TagImageGridAgent to verify that it works correctly
	'''

	@classmethod
	def setUpClass(self):
		'''
		Creates a d3cryp7TagImageGridAgent to test
		'''

		self.agent = d3cryp7TagImageGridAgent('http://localhost')

	def test_init(self):
		'''
		Tests that the d3cryp7TagImageGridAgent initializes correctly
		'''

		self.assertEqual(self.agent._cost, 0.0)
		self.assertIsInstance(self.agent._cost, float)

	def test_solve(self):
		'''
		Tests that the d3cryp7TagImageGridAgent solves a Challenge correctly
		'''

		if TRUCKS:
			data = {
				'image': TRUCKS,
				'tag': 'truck',
				'columns': 3,
				'rows': 3
			}

			self.agent.solve(data)
			self.assertEqual(type(data['result']), tuple)

			data = {
				'image': None,
				'tag': '',
				'columns': 3,
				'rows': 3
			}

			with self.assertRaises(RuntimeError):
				self.agent.solve(data)

			with self.assertRaises(TypeError):
				self.agent.solve(None)
		else:
			self.fail('Could not locate data file')

	def test_cost(self):
		'''
		Tests that the d3cryp7TagImageGridAgent returns the correct cost
		'''

		self.assertEqual(self.agent.get_cost(), 0.0)
		self.assertIsInstance(self.agent.get_cost(), float)

	def test_comparable(self):
		'''
		Tests that a d3cryp7TagImageGridAgent can be compared to another Agent
		'''

		self.assertFalse(self.agent < self.agent)

		with self.assertRaises(TypeError):
			self.agent < None

	def test_success(self):
		'''
		Tests that a d3cryp7TagImageGridAgent can handle a successful Challenge
		correctly
		'''

		if TRUCKS:
			data = {
				'image': TRUCKS
			}

			self.agent.success(data)
		else:
			self.fail('Could not locate data file')

	def test_fail(self):
		'''
		Tests that a d3cryp7TagImageGridAgent can handle a failed Challenge correctly
		'''

		if TRUCKS:
			data = {
				'image': TRUCKS
			}

			self.agent.fail(data)
		else:
			self.fail('Could not locate data file')

if __name__ == '__main__':
	unittest.main()
